'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var apolloDatasourceRest = require('apollo-datasource-rest');
var jwt = _interopDefault(require('jsonwebtoken'));
var isEmpty = _interopDefault(require('lodash/isEmpty'));
var jwksClient = _interopDefault(require('jwks-rsa'));

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    if (enumerableOnly) symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    });
    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(Object(source), true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(Object(source)).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

function _defineProperty$1(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}
class ServerClient extends apolloDatasourceRest.RESTDataSource {
  constructor(props) {
    super();

    _defineProperty$1(this, "managementToken", async () => {
      try {
        if (!this.tokenExpired) return this._managementToken;
        const data = await this.post('/oauth/token', {
          client_id: this.props.clientId,
          client_secret: this.props.clientSecret,
          audience: `https://${this.props.domain}/api/v2/`,
          grant_type: 'client_credentials'
        }, {
          cacheOptions: {
            ttl: 0
          }
        });
        this._managementToken = data.access_token;
        this._tokenExpiresAt = Date.now() + data.expires_in * 1000;
        return this._managementToken;
      } catch (err) {
        var _err$response, _err$response$data;

        throw (err === null || err === void 0 ? void 0 : (_err$response = err.response) === null || _err$response === void 0 ? void 0 : (_err$response$data = _err$response.data) === null || _err$response$data === void 0 ? void 0 : _err$response$data.message) || (err === null || err === void 0 ? void 0 : err.message) || err;
      }
    });

    this.props = _objectSpread2({
      ttl: 60,

      willSendRequest() {}

    }, props);
    if (!this.props.domain) throw new Error('Auth0 ServerClient domain is required');
    if (!this.props.clientId) throw new Error('Auth0 ServerClient management clientId is required');
    if (!this.props.clientSecret) throw new Error('Auth0 ServerClient management clientSecret is required');
    this.baseURL = `https://${this.props.domain}`;
  }

  async willSendRequest(request) {
    try {
      // Prevent an infinite loop (caused by the management token request)
      if (request.path === '/oauth/token') return;
      const managementToken = await this.managementToken();
      request.headers.set('Authorization', `Bearer ${managementToken}`);
      this.props.willSendRequest(request);
    } catch (err) {
      throw err;
    }
  }

  get tokenExpired() {
    if (!this._tokenExpiresAt) return true;
    return Date.now() > this._tokenExpiresAt;
  }

  getUsers(params) {
    return this.get('/api/v2/users', _objectSpread2({}, params, {
      search_engine: 'v3'
    }), {
      cacheOptions: {
        ttl: this.props.ttl
      }
    });
  }

  getUser(id, params) {
    return this.get(`/api/v2/users/${id}`, _objectSpread2({}, params, {
      search_engine: 'v3'
    }), {
      cacheOptions: {
        ttl: this.props.ttl
      }
    });
  }

  getUsersByEmail(params) {
    return this.get('/api/v2/users-by-email', _objectSpread2({}, params, {
      search_engine: 'v3'
    }), {
      cacheOptions: {
        ttl: this.props.ttl
      }
    });
  }

  async createUser(user) {
    try {
      const res = await this.post('/api/v2/users', _objectSpread2({
        connection: 'Username-Password-Authentication'
      }, user));
      return res;
    } catch (err) {
      var _err$extensions, _err$extensions$respo, _err$extensions$respo2;

      throw (err === null || err === void 0 ? void 0 : (_err$extensions = err.extensions) === null || _err$extensions === void 0 ? void 0 : (_err$extensions$respo = _err$extensions.response) === null || _err$extensions$respo === void 0 ? void 0 : (_err$extensions$respo2 = _err$extensions$respo.body) === null || _err$extensions$respo2 === void 0 ? void 0 : _err$extensions$respo2.message) || err;
    }
  }

  async resetPassword(id, params) {
    try {
      const {
        newPassword
      } = params;
      const res = await this.patch(`/api/v2/users/${id}`, {
        password: newPassword,
        connection: 'Username-Password-Authentication'
      }, {
        cacheOptions: {
          ttl: 0
        }
      });
      return res;
    } catch (err) {
      var _err$extensions2, _err$extensions2$resp, _err$extensions2$resp2;

      throw (err === null || err === void 0 ? void 0 : (_err$extensions2 = err.extensions) === null || _err$extensions2 === void 0 ? void 0 : (_err$extensions2$resp = _err$extensions2.response) === null || _err$extensions2$resp === void 0 ? void 0 : (_err$extensions2$resp2 = _err$extensions2$resp.body) === null || _err$extensions2$resp2 === void 0 ? void 0 : _err$extensions2$resp2.message) || err;
    }
  }

  sendChangePasswordEmail(email) {
    return this.post('/dbconnections/change_password', {
      email,
      connection: 'Username-Password-Authentication'
    }, {
      cacheOptions: {
        ttl: 0
      }
    });
  }

}

let _jwskClient;

var getKey = (domain => {
  if (isEmpty(domain)) throw new Error('apollo-auth0 getKey requires domain');

  if (!_jwskClient) {
    _jwskClient = jwksClient({
      jwksUri: `https://${domain}/.well-known/jwks.json`
    });
  }

  return function getKey(header, cb) {
    _jwskClient.getSigningKey(header.kid, (err, key) => {
      if (err) return cb(err);
      const signingKey = key.publicKey || key.rsaPublicKey;
      return cb(null, signingKey);
    });
  };
});

function currentUser(token, options = {}) {
  if (!options.domain) throw new Error('Auth0 currentUser domain is required');
  if (!options.clientId) throw new Error('Auth0 currentUser clientId is required');
  if (isEmpty(token)) return null;
  return new Promise(resolve => {
    jwt.verify(token, getKey(options.domain), _objectSpread2({
      algorithms: ['RS256'],
      ignoreExpiration: false
    }, options, {
      audience: options.clientId,
      issuer: `https://${options.domain}/`
    }), (err, decoded) => {
      if (err) return resolve();
      return resolve(_objectSpread2({
        id: decoded.sub
      }, decoded));
    });
  });
}

exports.ServerClient = ServerClient;
exports.currentUser = currentUser;
//# sourceMappingURL=index.js.map
